/***************************************************************************
                          klcc.cpp  -  klcc mainwindow
                             -------------------
    begin                : fre mar  9 20:53:57 CET 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// App headers
#include "klcc.h"
#include "lcpio/misc.h"
#include "lcpio/lcpline.h"
#include "klpreferencesdlg.h"
#include "klsettings.h"

// Qt headers
#include <qstringlist.h>
#include <qptrlist.h>

// KDE headers
#include <kstdaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpassdlg.h>
#include <knotifyclient.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <knotifydialog.h>

#include <kwallet.h>

KLCC::KLCC(QWidget*, const char *name)
	: KMainWindow(0, name), mExiting(false)
{
	mWallet = 0;
	config = kapp->config();
	setCaption(i18n("Default"));

	lcpIO = new LcpIO(this, "lcpIO");
	beepTimer = new QTimer(this, "beepTimer");
	dcop = new KLDcop(this, "dcopInterface");
	serverSettings = new KLServerSettings();
	lookNFeelSettings = new KLLookNFeelSettings();

	initStatusbar();
	initAction();
	initView();
	initDockWindow();
	loadOptions();

	// Connect signals and slots
	// A change in clientstatus
	connect(lcpIO, SIGNAL(clientstatusChanged(int)), this, SLOT(slotClientstatus(int)));
	connect(lcpIO, SIGNAL(clientstatusChanged(int)), view, SLOT(setClientstatus(int)));
	connect(lcpIO, SIGNAL(clientstatusChanged(int)), trayIcon, SLOT(slotClientstatus(int)));
	connect(lcpIO, SIGNAL(clientstatusChanged(int)), dcop, SLOT(clientstatusChanged(int)));
	// New info about the line
	connect(lcpIO, SIGNAL(selectedLineInfoUpdate(t_lcp3_info_tput, double, double)),
		view, SLOT(setInfo(t_lcp3_info_tput, double, double)));
	connect(lcpIO, SIGNAL(selectedLineInfoUpdate(t_lcp3_info_tput, double, double)),
		trayIcon, SLOT(setTooltip(t_lcp3_info_tput)));
	connect(lcpIO, SIGNAL(selectedLineAutoOffline(t_lcp3_info_tput)),
		this, SLOT(slotOffline()));

	// A change in linestatus
	connect(lcpIO, SIGNAL(selectedLineStatusChanged(int)), this, SLOT(slotLinestatus(int)));
	connect(lcpIO, SIGNAL(selectedLineStatusChanged(int)), view, SLOT(setLinestatus(int)));
	connect(lcpIO, SIGNAL(selectedLineStatusChanged(int)), trayIcon, SLOT(slotLinestatus(int)));
	// Show num of lines that are online on the server
	connect(lcpIO, SIGNAL(numLinesOnlineChanged(int)), this, SLOT(setNumLinesOnline(int)));
	// The connection to the server was lost!
	connect(lcpIO, SIGNAL(connectionDown()), this, SLOT(slotConnectionLost()));
	// Server didn't allow client
	connect(lcpIO, SIGNAL(badAuth()), this, SLOT(slotBadAuth()));
	// Client has done something forbidden
	connect(lcpIO, SIGNAL(accesserror(int)), this, SLOT(slotNoAccess(int)));
	// Some ISDN magic :)
	connect(lcpIO, SIGNAL(chanFail()), this, SLOT(slotChanFail()));
	connect(lcpIO, SIGNAL(isdnAct(t_lcp3_info_isdn)), this, SLOT(slotIsdnAct(t_lcp3_info_isdn)));
	// Serverstatus has been received
	connect(lcpIO, SIGNAL(serverStatus(t_lcp3_info_status)), this, SLOT(slotServerstatus(t_lcp3_info_status)));
	// All lines received so we can update the linename now
	connect(lcpIO, SIGNAL(lineChanged(const QString &)), this, SLOT(changeLineSettings(const QString &)));
	connect(lcpIO, SIGNAL(lineChanged(const QString &)), view, SLOT(setLinename(const QString &)));
	// Num of channels used
	connect(lcpIO, SIGNAL(selectedLineChanChanged(int)), this, SLOT(slotChannels(int)));
	connect(lcpIO, SIGNAL(selectedLineChanChanged(int)), trayIcon, SLOT(slotChannels(int)));
	// beepTimer timeout
	connect(beepTimer, SIGNAL(timeout()), this, SLOT(beep()));
	// Client is going offline
	connect(lcpIO, SIGNAL(clientOffline()), this, SLOT(slotOfflineNotify()));

	// Check if we should connect to server on startup
	if(serverSettings->connectOnStartup())
		QTimer::singleShot(0, this, SLOT(slotConnect()));

	// If automatically go online after connect
	lcpIO->setAutoOnline(lookNFeelSettings->autoOnline());

	view->setCostCalculationActive(lcpIO->selectedLine()->costInfo().costCalcActive);
}

KLCC::~KLCC()
{
	slotCloseWallet();
	delete serverSettings;
	delete lookNFeelSettings;
}

int KLCC::currentLine()
{
	return lcpIO->selectedLine()->lineId();
}

void KLCC::initAction()
{
	KActionCollection *ac = actionCollection();

	KStdAction::quit(this, SLOT(quitKLCC()), ac);
	connectAction = new KAction(i18n("&Connect"), "up", CTRL+Key_Up,
		this, SLOT(slotConnect()), ac, "serverConnect");
	disconnectAction = new KAction(i18n("&Disconnect"), "down", CTRL+Key_Down,
		this, SLOT(slotDisconnect()), ac, "serverDisconnect");
	serverstatusAction = new KAction(i18n("&Status..."), 0,
		lcpIO, SLOT(getServerStatus()), ac, "serverStatus");
	shutdownAction = new KAction(i18n("&Shutdown"), 0,
		lcpIO, SLOT(shutdownServer()), ac, "serverShutdown");
	onlineAction = new KAction(i18n("&Online"), "connect_established", ALT+Key_Up,
		this, SLOT(slotOnline()), ac, "clientOnline");
	offlineAction = new KAction(i18n("O&ffline"), "connect_no", ALT+Key_Down,
		this, SLOT(slotOffline()), ac, "clientOffline");
	disableAutoOfflineAction = new KToggleAction(i18n("&Disable Auto-offline"),
		0, this, SLOT(slotAutoOfflineDisabled()), ac, "clientDisableAutoOffline");
	lockAction = new KAction(i18n("Lock &line"), "lock", 0,
		this, SLOT(slotLock()), ac, "clientLock");
	selectAction = new KAction(i18n("&Select line..."), 0,
		this, SLOT(slotLineSelection()), ac, "clientLineSelect");
	channelAction = new KAction(i18n("&Use 2 ISDN channels"), 0,
		this, SLOT(slotToggleChan()), ac, "clientChannel");

	createStandardStatusBarAction();

	setStandardToolBarMenuEnabled(true);

	menubarAction = KStdAction::showMenubar(this, SLOT(showMenubar()), ac);
	preferencesAction = KStdAction::preferences(this, SLOT(slotSettings()), ac);

	KStdAction::keyBindings(this, SLOT(slotKeyBindings()), ac);
#if KDE_IS_VERSION(3,2,90)
	KStdAction::configureToolbars(this, SLOT(configureToolbars()), ac);
#else
	KStdAction::configureToolbars(this, SLOT(slotConfigToolbars()), ac);
#endif
	KStdAction::configureNotifications(this, SLOT(slotConfigNotifications()), ac);

	createGUI();

	slotClientstatus(CLT_DISCON); // startup status is disconnected
}

void KLCC::initView()
{
	view = new KLView(this, "mainView");
	setCentralWidget(view);
}

void KLCC::initStatusbar()
{
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG, 1, false);
  statusBar()->setItemAlignment(ID_STATUS_MSG, AlignLeft|AlignVCenter);
  statusBar()->insertItem(QString::null, ID_CONNECTION_MSG, 0, true);
  statusBar()->insertItem(QString::null, ID_LINES_MSG, 0, true);
  setNumLinesOnline(0);
}

void KLCC::initDockWindow()
{
  trayIcon = new KLDockWindow(this, "trayIcon");
  connectAction->plug(trayIcon->menu());
  disconnectAction->plug(trayIcon->menu());
  trayIcon->menu()->insertSeparator();
  onlineAction->plug(trayIcon->menu());
  offlineAction->plug(trayIcon->menu());
  disableAutoOfflineAction->plug(trayIcon->menu());
  trayIcon->menu()->insertSeparator();
  lockAction->plug(trayIcon->menu());
  trayIcon->menu()->insertSeparator();
  selectAction->plug(trayIcon->menu());
  trayIcon->menu()->insertSeparator();
  channelAction->plug(trayIcon->menu());
  trayIcon->menu()->insertSeparator();
  preferencesAction->plug(trayIcon->menu());
  connect(trayIcon, SIGNAL(quitSelected()), this, SLOT(quitKLCC()));
}

void KLCC::saveOptions()
{
  lookNFeelSettings->saveSettings(config);

  //toolBar("mainToolBar")->saveSettings(config, "ToolBar Settings");
  saveMainWindowSettings(config, "General Options");

  config->setGroup("General Options");
  if(lookNFeelSettings->saveWinPos())
    config->writeEntry("Position", pos());
  config->writeEntry("Geometry", size());

  if(lookNFeelSettings->saveWinState())
  {
    if(isMinimized())
      config->writeEntry("State", "Minimized");
    else if(isHidden())
      config->writeEntry("State", "Hidden");
    else
      config->writeEntry("State", "Shown");
  }

  serverSettings->saveSettings(config);

  config->setGroup("Client Options");

  if(lookNFeelSettings->saveLastLine())
    config->writeEntry("Line", lcpIO->selectedLine()->lineId());

  config->sync();
}

void KLCC::loadOptions()
{
	toolBar("mainToolBar")->applySettings(config, "ToolBar Settings");

	applyMainWindowSettings( config, "General Options" );

	serverSettings->loadSettings(config);
	lookNFeelSettings->loadSettings(config);

	lcpIO->setAutoReconnect(
		serverSettings->reconnect(),
		serverSettings->reconnectDelay(),
		serverSettings->reconnectCount());

	if(lookNFeelSettings->dock())
		trayIcon->show();
	else
		trayIcon->hide();

	lcpIO->setAutoRedial(lookNFeelSettings->redial(),
		lookNFeelSettings->redialDelay(), lookNFeelSettings->redialCount());

	config->setGroup("Client Options");

	if(lookNFeelSettings->saveLastLine())
	{
		int tmp = config->readNumEntry("Line", 0);
		lcpIO->setSelectedLine(tmp);
	}

	config->setGroup("General Options");
	QSize size = config->readSizeEntry("Geometry");
	if(!size.isEmpty())
		resize(size);

	if(lookNFeelSettings->saveWinPos())
	{
		QPoint pos = config->readPointEntry("Position");
		move(pos);
	}

	if(lookNFeelSettings->saveWinState())
	{
		QString tmp = config->readEntry("State", "Shown");
		if(tmp == "Minimized")
			showMinimized();
		else if(tmp == "Hidden")
			hide();
		else
			show();
	}
	else
	{
		show();
	}

	//Failsafe
	if(isHidden() && trayIcon->isHidden())
		show();

	menubarAction->setChecked( !menuBar()->isHidden() );
}

bool KLCC::queryExit() {
  saveOptions();
  return true;
}

void KLCC::slotClientstatus(int status)
{
	switch(status)
	{
		case CLT_DISCON:
		{
			connectAction->setEnabled(true);
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(false);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(false);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(false);
			slotStatusMsg(i18n("Disconnected"), ID_CONNECTION_MSG);
			break;
		}

		case CLT_NOAUTH:
		{
			connectAction->setEnabled(false);
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(false);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(false);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(false);
			break;
		}

		case CLT_OFFLINE:
		{
			connectAction->setEnabled(false);
			disconnectAction->setEnabled(true);
			serverstatusAction->setEnabled(true);
			onlineAction->setEnabled(true);
			offlineAction->setEnabled(false);
			selectAction->setEnabled(true);
			shutdownAction->setEnabled(true);
			channelAction->setEnabled(false);

			beepTimer->stop();
			slotStatusMsg(i18n("Connected"), ID_CONNECTION_MSG);
			break;
		}

		case CLT_DIALING:
		{
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(true);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(true);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(false);
			break;
		}

		case CLT_ONLINE:
		{
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(true);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(true);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(true);

			// Minimize when client is online
			if(lookNFeelSettings->minimizeOnline())
			{
				if(trayIcon->isHidden())
					showMinimized();
				else
					hide();
			}

			KNotifyClient::event(winId(), "notifyonline", i18n("Online"));

			// Start beepTimer if active
			beepTimer->start((int)(lookNFeelSettings->onlineWarningDelay() * 60000), false);

			// Exec ONLINE-SCRIPT
			if(lcpIO->selectedLine()->onlineScriptActive())
			{
				disconnect(&m_script, SIGNAL(processExited(KProcess*)),
					this, SLOT(slotOfflineScriptReady()));

				m_script.clearArguments();
				m_script << KShellProcess::quote(lcpIO->selectedLine()->onlineScript());
				m_script.start();
			}
			break;
		}

		case CLT_CLOSING:
		{
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(true);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(false);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(false);
			break;
		}

		case CLT_ABORTING:
		{
			disconnectAction->setEnabled(false);
			serverstatusAction->setEnabled(true);
			onlineAction->setEnabled(false);
			offlineAction->setEnabled(false);
			selectAction->setEnabled(false);
			shutdownAction->setEnabled(false);
			channelAction->setEnabled(false);
			break;
		}
	}
}

void KLCC::slotLinestatus(int status)
{
	switch(status)
	{
		case CST_DOWN:
		case CST_DIALING:
		case CST_UP_USER:
		case CST_CLOSING:
		case CST_CLOSE_ERROR:
		case CST_LOCKED_UP_TIMED:
		case CST_LOCKED_DOWN:
			lockAction->setEnabled(false);
			break;
		case CST_UP_SERVER:
			lockAction->setText(i18n("Lock &line"));
			lockAction->setIcon("lock");
			lockAction->setEnabled(true);
			break;
		case CST_LOCKED_UP_CLT:
			lockAction->setText(i18n("Unlock &line"));
			lockAction->setIcon("unlock");
			lockAction->setEnabled(true);
			break;
	}
}

void KLCC::slotConnect()
{
	if(serverSettings->authenticate())
	{
		if (serverSettings->savePasswd())
		{
			// connect using password from kwallet

			slotCloseWallet(); // just to make sure we left no wallet object around
			// try to open it, this may fail if the user
			// has disabled kwallet altogether
			mWallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(),
				winId(), KWallet::Wallet::Asynchronous);

			if (mWallet)
			{
				// TODO: can we determine if this waiting ever ends?
				connect(mWallet, SIGNAL(walletOpened(bool)),
					this, SLOT(slotConnectWithWallet()));

				// make sure we find out about the wallet being closed,
				// this might be caused by a timed close for instance
				connect(mWallet, SIGNAL(walletClosed()), this, SLOT(slotCloseWallet()));
			}
			else
			{
				// KWallet disabled by user, we have to ask for the password
				connectWithPasswordRequest();
			}
		}
		else
		{
			// connect using password provided by user input
			connectWithPasswordRequest();
		}
	}
	else
	{
		// connect without authentication
		slotStatusMsg(i18n("Connecting..."), ID_CONNECTION_MSG);

		lcpIO->connectToServer(
			serverSettings->host(),
			serverSettings->port(),
			QString(""), // username
			QString("") // password
			);
	}
}


void KLCC::connectWithPasswordRequest()
{
	QCString tempPasswd;

	int result = KPasswordDialog::getPassword(tempPasswd,
		i18n("Authenticate with server"));
	if (result == KPasswordDialog::Accepted)
	{
		slotStatusMsg(i18n("Connecting..."), ID_CONNECTION_MSG);
		lcpIO->connectToServer(
			serverSettings->host(),
			serverSettings->port(),
			serverSettings->user(),
			tempPasswd);
	}
	else
	{
		slotStatusMsg(i18n("Connecting aborted, no password provided by user"),
			ID_CONNECTION_MSG);
	}
}


void KLCC::slotConnectWithWallet()
{
	QCString tempPasswd;
	int result = 0; // used for password input dialogs
	bool gotPassword = false; // flag to show if tempPasswd contents are valid

	if (KWallet::Wallet::isOpen(KWallet::Wallet::NetworkWallet()))
	{
		// password should be saved in the wallet
		if(!mWallet->hasFolder("KLCC"))
			mWallet->createFolder("KLCC");

		if(mWallet->setFolder("KLCC"))
		{
			QString walletPass;

			int ret = mWallet->readPassword(QString::fromLatin1("Password"),
				walletPass);
			gotPassword = (ret == 0);

			if (gotPassword)
			{
				// Successfully read password from wallet
				tempPasswd = walletPass.latin1();
			}
			else
			{
				// Password entry not found in wallet

				result = KPasswordDialog::getPassword(tempPasswd,
					i18n("Authenticate with server"));
				if (result == KPasswordDialog::Accepted)
				{
					// new password typed in by user, store it in the wallet
					ret = mWallet->writePassword(QString::fromLatin1("Password"),
						QString::fromLatin1(tempPasswd));
					gotPassword = (ret == 0);
				}

				if (!gotPassword)
				{
					// No password in wallet, none typed in or saving to wallet failed, abort connecting
					slotStatusMsg(i18n("Connecting failed, no password provided by user or wallet"), ID_CONNECTION_MSG);
				}
			}
		} // END if(mWallet->setFolder() ... )
		else
		{
			// could not switch to our folder, bad bad
			kdWarning() << "Could not switch to folder KLCC in wallet" << endl;
		}
	} // END if (wallet is open)

	slotCloseWallet();

	if (gotPassword)
	{
		slotStatusMsg(i18n("Connecting..."), ID_CONNECTION_MSG);
		lcpIO->connectToServer(
			serverSettings->host(),
			serverSettings->port(),
			serverSettings->user(),
			tempPasswd);
	}
	else
	{
		// fallback
		connectWithPasswordRequest();
	}
}

void KLCC::slotCloseWallet()
{
	delete mWallet;
	mWallet = 0;
}

void KLCC::slotDisconnect()
{
	lcpIO->disconnectFromServer();
}

void KLCC::slotOnline()
{
	lcpIO->online();
}

void KLCC::slotOffline()
{
	if(lcpIO->selectedLine()->offlineScriptActive())
	{
		connect(&m_script, SIGNAL(processExited(KProcess*)), this,
			SLOT(slotOfflineScriptReady()));
		m_script.clearArguments();
		m_script << KShellProcess::quote(lcpIO->selectedLine()->offlineScript());
		m_script.start();
	}
	else
	{
		lcpIO->offline();
	}
}

void KLCC::slotLock()
{
	lcpIO->toggleLineLock();
}

void KLCC::slotLineSelection()
{
	int cur = 0;
	QPtrList<LcpLine>* lines;
	KLLineDlg lineDlg(this);

	// Show lineselectiondialog
	lines = lcpIO->lineList();

	for(uint i = 1; i < lines->count(); i++)
	{
		lineDlg.addLine(lines->at(i)->name());
		if(lcpIO->selectedLine() == lines->at(i))
			cur = i - 1;
	}

	lineDlg.setCurrentLine(cur);

	if(lineDlg.exec())
	{
		lcpIO->setSelectedLine(lines->at(lineDlg.selectedLine() + 1)->lineId());
		setCaption(lcpIO->selectedLine()->name());
		view->setLinename(lcpIO->selectedLine()->name());
		dcop->lineChanged(lcpIO->selectedLine()->lineId());
		slotLinestatus(lcpIO->selectedLine()->status());
		view->setLinestatus(lcpIO->selectedLine()->status());
		trayIcon->slotLinestatus(lcpIO->selectedLine()->status());
	}
}

void KLCC::slotSettings()
{
	slotStatusMsg(i18n("Setup KLCC..."));

	KLPreferencesDlg prefDlg(this);

	// Server //
	prefDlg.addServerSettings(*(serverSettings));

	// Look & Feel //
	prefDlg.addLookNFeelSettings(*(lookNFeelSettings));

	// Line //
	QPtrList<LcpLine>* lines = lcpIO->lineList();
	uint numLines = lines->count();

	for(uint i = 0; i < numLines; i++)
		prefDlg.addLine(*(lines->at(i)));

	if(prefDlg.exec())
	{
		// Server //
		prefDlg.getServerSettings(*(serverSettings));

		// Look & Feel //
		prefDlg.getLookNFeelSettings(*(lookNFeelSettings));

		// Line //
		for(uint i = 0; i < numLines; i++)
			prefDlg.getLine(i, *(lines->at(i)));
	}

	slotStatusMsg(i18n("Ready."));
}

void KLCC::showMenubar()
{
	if(menubarAction->isChecked())
		menuBar()->show();
	else
		menuBar()->hide();
}

void KLCC::slotStatusMsg(const QString &text, int id)
{
	statusBar()->clear();
	statusBar()->changeItem(text, id);
}

void KLCC::slotConnectionLost()
{
	KNotifyClient::event(winId(), "error",
		i18n("Lost the connection to the server"));
}

void KLCC::slotBadAuth()
{
	KNotifyClient::event(winId(), "error", i18n("Authentication failed!"));
}

void KLCC::slotNoAccess(int cmd) {
	QString text, cause;

	switch(cmd)
	{
	case CMD3_LOCK:
		cause = i18n("You may not lock this line.");
		break;
	case CMD3_UNLOCK:
		cause = i18n("You may not unlock this line.");
		break;
	case CMD3_SHUTDOWN:
		cause = i18n("You may not shutdown server.");
		break;
	default:
		cause = i18n("You have no access to command %1.").arg(cmd);
		break;
	}

	text = i18n("Permission denied\n%1").arg(cause);
	KNotifyClient::event(winId(), "error", text);
}

void KLCC::slotChanFail()
{
	channelAction->setEnabled(true);
	KNotifyClient::event(winId(), "error", i18n("Manipulating ISDN channels failed!"));
}

void KLCC::slotIsdnAct(t_lcp3_info_isdn info)
{
	QString text = i18n("Caller: %1\nCall to: %2\nTime: %3 %4").
		arg(info.caller).arg(info.on).arg(info.time).
		arg(LCPIO::formatDate(info.year, info.month, info.day));

	KNotifyClient::event(winId(), "notifyincomming", text);
}

void KLCC::slotServerstatus(t_lcp3_info_status status)
{
	QString text = i18n("Connected clients: %1\nClients online: %2").arg(
		status.clients).arg(status.online);
	KMessageBox::information(this, text, i18n("Serverstatus"));
}

void KLCC::slotChannels(int num)
{
	channelAction->setEnabled(true);
	channelAction->setText(i18n("&Use 1 ISDN channel", "&Use %n ISDN channels", num));
	view->setTwoChan(num != 1);
}

void KLCC::beep()
{
	KNotifyClient::event(winId(), "notifytime");
}

void KLCC::slotToggleChan()
{
	lcpIO->toggleISDNChan();
	channelAction->setEnabled(false);
}

void KLCC::setNumLinesOnline(int lines)
{
	slotStatusMsg(i18n("1 Line", "%n Lines", lines), ID_LINES_MSG);
}

void KLCC::changeLineSettings(const QString& name)
{
	bool use = lcpIO->selectedLine()->costInfo().costCalcActive;
	view->setCostCalculationActive(use);
	setCaption(name);
}

void KLCC::slotOfflineNotify()
{
	KNotifyClient::event(winId(), "notifyoffline", i18n("Offline"));
}

void KLCC::slotAutoOfflineDisabled()
{
	bool mode = !LcpLine::autoOfflineDisabled();
	LcpLine::setAutoOfflineDisabled(mode);
	disableAutoOfflineAction->setChecked(mode);
}

void KLCC::slotOfflineScriptReady()
{
	lcpIO->offline();
}

void KLCC::slotKeyBindings()
{
	KKeyDialog::configure(actionCollection(), xmlFile());
}

void KLCC::slotConfigToolbars()
{
#if !KDE_IS_VERSION(3,2,90)
	saveMainWindowSettings(KGlobal::config(), "General Options");
	KEditToolbar dlg(actionCollection());
	connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
	dlg.exec();
#endif
}

void KLCC::slotNewToolbarConfig()
{
#if !KDE_IS_VERSION(3,2,90)
	createStandardStatusBarAction();
	createGUI();
	applyMainWindowSettings(KGlobal::config(), "General Options");
#endif
}

void KLCC::slotConfigNotifications()
{
	KNotifyDialog::configure(this, "notifyDialog");
}

void KLCC::closeEvent(QCloseEvent* e)
{
	if(mExiting)
	{
		KMainWindow::closeEvent(e);
		return;
	}

	KMessageBox::information( this,
		i18n("<qt>Closing the main window will keep KLCC running in the "
		"system tray. Use Quit from the File menu to quit the "
		"application.</qt>"), i18n("Docking in System Tray"),
		"hideOnCloseInfo");
	hide();
	e->ignore();
}

void KLCC::quitKLCC()
{
	if(!mExiting)
		mExiting = true;
	close();
}

#include "klcc.moc"
