/***************************************************************************
                          lcpio.cpp  -  description
                             -------------------
    begin                : Mon Feb 12 2001
    copyright            : (C) 2001 by Peter Simonsson
    email                : psn@linux.se
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Standard headers
#include <stdlib.h>

// Qt headers
#include <qstring.h>
#include <qtimer.h>

// KDE headers
#include <kdebug.h>
#include <kextsock.h>

// App headers
#include "lcpio.h"
#include "lcpline.h"
#include "sendqpost.h"

LcpIO::LcpIO(QObject* parent, const char* name) : QObject(parent, name) {
  // Socket
  m_socket = 0;
  connect(this, SIGNAL(cmdTimedOut()), this, SLOT(connectionLost()));
  // init sendQ
  m_sendQ = new QPtrList<SendQPost>();
  m_sendQ->setAutoDelete(true);
  // init line list
  m_lines = new QPtrList<LcpLine>();
  m_lines->setAutoDelete(true);
  LcpLine* tmp = new LcpLine("Default");
  tmp->setLineId(0);
  m_lines->append(tmp);
  m_selectedLine = m_lines->at(0);
  // Create buffer for incoming packets
  m_inBuffer = (char*) malloc(get_max_lcp3_pack_size() * 3);
  m_received = 0;
  // AutoOnline
  m_autoOnline = false;
  // Line handling
  m_savedLineId = 0;
  m_allLinesReceived = false;
  // ISND channels handling
  m_waitChanUpdate = false;
  // Auto reconnect
  m_autoReconnectEnabled = false;
  m_autoReconnectDelay = 0;
  m_autoReconnectCount = 0;
  m_autoReconnectTries = 0;
  m_autoReconnectTimer = new QTimer(this, "autoReconnectTimer");
  connect(m_autoReconnectTimer, SIGNAL(timeout()), this, SLOT(reconnect()));
  // Auto redial
  m_autoRedialEnabled = false;
  m_autoRedialDelay = 0;
  m_autoRedialCount = 0;
  m_autoRedialTries = 0;
  m_autoRedialTimer = new QTimer(this, "autoRedialTimer");
  connect(m_autoRedialTimer, SIGNAL(timeout()), this, SLOT(redial()));
  m_allowOffline = true;
  // Send timers
  m_resendTimer = new QTimer(this, "resendTimer");
  m_keepAliveTimer = new QTimer(this, "keepAliveTimer");
  connect(m_resendTimer, SIGNAL(timeout()), this, SLOT(processSendQ()));
  connect(m_keepAliveTimer, SIGNAL(timeout()), this, SLOT(keepAlive()));
  // Client Status
  m_clientStatus = CLT_DISCON;
  // Wait for all lines to be received before we do anything else
  connect(this, SIGNAL(allLinesReceived()), this, SLOT(postConnectProcessing()));
}

LcpIO::~LcpIO() {
  if(m_socket != 0) {
    disconnectFromServer();
  }

  cleanup();
  delete m_sendQ;
  delete m_inBuffer;
}

QPtrList<LcpLine>* LcpIO::lineList() const {
  return m_lines;
}

LcpLine* LcpIO::selectedLine() const {
  return m_selectedLine;
}

void LcpIO::setAutoReconnect(bool use, int delay, int count) {
  m_autoReconnectEnabled = use;
  m_autoReconnectDelay = delay;
  m_autoReconnectCount = count;
}

void LcpIO::setAutoRedial(bool use, int delay, int count) {
  m_autoRedialEnabled = use;
  m_autoRedialDelay = delay;
  m_autoRedialCount = count;
}

void LcpIO::processCmd(void* incoming) {
  struct t_lcp3_cmd* packet = static_cast<struct t_lcp3_cmd*>(incoming);
  void* infoPtr = static_cast<char*>(incoming) + sizeof(struct t_lcp3_cmd);

  // Debug
  //kdDebug() << "Command: " << packet->cmd << endl;

  // Send ack?
  if((packet->cmd < MIN_CBR3) && (packet->cmd != CMD3_ACK) && (packet->cmd != CMD3_UNREG)) {
     //kdDebug() << "Send ack for: " << packet->cmd << endl;
     this->sendAck(packet->cmd);
  }

  if(packet->info_is_valid) {
    // connecting to server or going offline?
    if((m_clientStatus == CLT_NOAUTH) && (packet->client_stat == CLT_OFFLINE)) {
      // We need lines!!
      addToSendQ(CMD3_LINESREQ); // Request lines from server
      m_autoReconnectTries = 0;
    } else if((m_clientStatus > CLT_OFFLINE) && (packet->client_stat <= CLT_OFFLINE)) {
      if(m_autoRedialEnabled && !m_allowOffline && (m_autoRedialTries < m_autoRedialCount)) {
        m_autoRedialTimer->start(m_autoRedialDelay * 1000, true);
      }

      emit clientOffline();
    }

    // Clientstatus update?
    if(m_clientStatus != packet->client_stat) {
      m_clientStatus = packet->client_stat;
      emit clientstatusChanged(m_clientStatus);

      if(m_clientStatus == CLT_ONLINE) {
        m_autoRedialTries = 0;
      }
    }

    if(m_allLinesReceived) {
      LcpLine* line = findLine(packet->line_id);

      // Linestatus update
      if(line->status() != packet->line_stat) {
        //kdDebug() << "Line status for line " << packet->line_id << " is " << packet->line_stat << endl;
        line->setStatus(packet->line_stat);
        emit lineStatusChanged(packet->line_stat, packet->line_id);

        // is it the selected line that changed status?
        if(m_selectedLine == line) {
          emit selectedLineStatusChanged(packet->line_stat);
        }

        emit numLinesOnlineChanged(LcpLine::numLinesOnline());
      }
    }
  }

  // process packet
  switch(packet->cmd) {
    // TCP connection gets dropped by server
    case CMD3_AUTHBAD:
      cleanup();
      emit badAuth();
    break;
    case CMD3_CLIENTSTAT:
      // Nothing is needed here!
    break;
    case CMD3_LINE: {
      LcpLine* line = new LcpLine(static_cast<t_lcp3_info_line*>(infoPtr)->linename, m_socket->host());
      line->setStatus(CST_DOWN);
      line->setLineId(static_cast<t_lcp3_info_line*>(infoPtr)->line_id);
      line->setMaxThroughputUp(static_cast<t_lcp3_info_line*>(infoPtr)->capability_baud_up);
      line->setMaxThroughputDown(static_cast<t_lcp3_info_line*>(infoPtr)->capability_baud_down);
      line->setNumChannels(1);
      m_lines->append(line);
      //kdDebug() << "Added line-id: " << static_cast<t_lcp3_info_line*>(infoPtr)->line_id << endl;

      if((m_lines->count() - 1) == static_cast<uint>(static_cast<t_lcp3_info_line*>(infoPtr)->total_lines)) {
        emit allLinesReceived();
      }
    }
    break;
    case CMD3_STATUS:
      emit serverStatus(*static_cast<t_lcp3_info_status*>(infoPtr));
      break;
    case CMD3_INVALID:
      emit invalidCommand(static_cast<t_lcp3_info_ack*>(infoPtr)->ackcmd);
    break;
    case CMD3_ENCINFO: {
      struct t_lcp3_info_auth* infoAuth = new struct t_lcp3_info_auth;
      infoAuth->enctype = static_cast<t_lcp3_info_enckey*>(infoPtr)->enctype;

      switch(infoAuth->enctype){
        /* These are unsupported at the moment
        case LCP3_ENCTYPE_SYM:
        case LCP3_ENCTYPE_ASYM: */
        // No authentication!
        case LCP3_ENCTYPE_OPEN:
          infoAuth->name[0] = 0;
          infoAuth->pass[0] = 0;
        break;
        // Plain text! (Default)
        case LCP3_ENCTYPE_NONE:
        default:
          qstrncpy(infoAuth->name, m_user.local8Bit(), LCP3_MAX_NAME_LEN);
          qstrncpy(infoAuth->pass, m_passwd.local8Bit(), LCP3_MAX_PWD_LEN);
        break;
      }

      addToSendQ(CMD3_AUTHREQ, infoAuth, sizeof(struct t_lcp3_info_auth));
      delete infoAuth;
      infoAuth = NULL;
    }
    break;
    case CMD3_NOACCESS:
      emit accesserror(static_cast<t_lcp3_info_ack*>(infoPtr)->ackcmd);
    break;
    case CMD3_CHANFAIL:
      m_waitChanUpdate = false;
      emit chanFail();
    break;
    case CMD3_ACK:
      //kdDebug() << "Ack for: " << static_cast<t_lcp3_info_ack*>(infoPtr)->ackcmd << endl;
      if(static_cast<t_lcp3_info_ack*>(infoPtr)->ackcmd != CMD3_REFRESH) {
         this->processAck(static_cast<t_lcp3_info_ack*>(infoPtr)->ackcmd);
      }
    break;
    // Disconnect from server
    case CMD3_UNREG:
      cleanup();
    break;
    case CBR3_TPUT:
	  {
      double clientCost = 0, lineCost = 0;
      LcpLine* line = findLine(packet->line_id);
		if(!line)
			break;
      line->calculateCost(*static_cast<t_lcp3_info_tput*>(infoPtr), clientCost, lineCost);

		if(m_selectedLine == line)
		{
			if(line->autoOffline(*static_cast<t_lcp3_info_tput*>(infoPtr)))
			{
				emit selectedLineAutoOffline(*static_cast<t_lcp3_info_tput*>(infoPtr));
				//offline();
			}

        emit selectedLineInfoUpdate(*static_cast<t_lcp3_info_tput*>(infoPtr), clientCost, lineCost);
      }

      emit lineInfoUpdate(*static_cast<t_lcp3_info_tput*>(infoPtr), clientCost, lineCost, packet->line_id);
    }
    break;
    case CBR3_LINESTAT: {
      // Num of ISDN channels used
      LcpLine* line = findLine(packet->line_id);
		if(!line)
			break;
      if(line->numChannels() != packet->line_channels) {
        line->setNumChannels(packet->line_channels);
        emit lineChanChanged(packet->line_channels);

        if(m_selectedLine == line) {
          emit selectedLineChanChanged(packet->line_channels);
          m_waitChanUpdate = false;
        }
      } else if(m_waitChanUpdate && (m_selectedLine == line)) {
        emit chanFail();
        m_waitChanUpdate = false;
      }
    }
    break;
    case CBR3_SRVDEATH:
      // The server has left the building... Time to clean up after the party!
      cleanup();

      if((m_autoReconnectTries <= m_autoReconnectCount) && m_autoReconnectEnabled) {
        m_autoReconnectTimer->start(m_autoReconnectDelay * 1000, true);
      } else {
        emit connectionDown();
      }
    break;
    case CBR3_ISDNACT:
      emit isdnAct(*static_cast<t_lcp3_info_isdn*>(infoPtr));
    break;
    // Unsupported commands
    default:
      //kdDebug() << "Command not yet supported! Command = " << packet->cmd << endl;
    break;
  }
}

void LcpIO::processAck(int cmd) {
  SendQPost *temp = 0;
  bool isFound = false;

  // Get first item in list
  temp = m_sendQ->first();
  //kdDebug() << "Processing ack..." << endl;

  while(temp && !isFound) { // Find first matching item
    if(temp->getCmd() == cmd) {
      if(!m_sendQ->remove()) {
        //kdDebug() << "Couldn't remove from queue: " << cmd << endl;
      }

      isFound = true;
    } else {
      temp = m_sendQ->next();
    }
  }

  if(!isFound) {
    //kdDebug() << "Didn't find command in queue: " << cmd << endl;
  } else {
    //kdDebug() << "Ok" << endl;
  }
}

void LcpIO::addToSendQ(int cmd, void* info, int infoSize) {
  SendQPost* temp = 0;

  temp = new SendQPost(cmd, info, infoSize);
  m_sendQ->append(temp);

  if(!temp->send(m_socket, m_selectedLine->lineId())) {
    emit cmdTimedOut();
    return;
  }

  // Activate resendTimer if it's inactive
  if(!m_resendTimer->isActive()) {
    m_resendTimer->start(RESENDINTERVAL);
  }

  // Deactivate keepAliveTimer if it's active
  if(m_keepAliveTimer->isActive()) {
    m_keepAliveTimer->stop();
  }
}

void LcpIO::sendAck(int cmd) {
  t_lcp3_info_ack infoAck;

  infoAck.ackcmd = cmd;
  // Create a temporary SendQPost so we can use send!!
  SendQPost temp(CMD3_ACK, &infoAck, sizeof(t_lcp3_info_ack));

  if(!temp.send(m_socket, m_selectedLine->lineId())) {
    emit cmdTimedOut();
    return;
  }
}

void LcpIO::cleanup() {
  //kdDebug() << "Cleaning up..." << endl;
  m_clientStatus = CLT_DISCON;
  emit clientstatusChanged(m_clientStatus);

  for(uint i = 1; i < (m_lines->count() - 1); i++) {
    emit lineStatusChanged(CST_DOWN, m_lines->at(i)->lineId());
  }

  emit selectedLineStatusChanged(CST_DOWN);
  emit numLinesOnlineChanged(0);
  m_resendTimer->stop();
  m_keepAliveTimer->stop();
  delete m_socket;
  m_socket = 0;
  m_sendQ->clear();
  m_lines->clear();
  LcpLine* tmp = new LcpLine("Default");
  tmp->setLineId(1);
  m_lines->append(tmp);
  m_selectedLine = m_lines->at(0);
  m_allLinesReceived = false;
  //kdDebug() << "Ready" << endl;
}

LcpLine* LcpIO::findLine(int lineId) {
  if(m_selectedLine->lineId() == lineId) {
    return m_selectedLine;
  }

  LcpLine* line = m_lines->first();

  while(line) {
    if(line->lineId() == lineId) {
      return line;
    }

    line = m_lines->next();
  }

  return 0;
}

void LcpIO::receiveData() {
  int size = 0;
  int packetsize = 0;
  int minsize = sizeof(t_lcp3_cmd);
  int available = get_max_lcp3_pack_size() * 3 - m_received; // Size of unused area
  char* ptr = (char *) m_inBuffer + m_received; // Pointer to unused area
  char* temp = 0;

  size = m_socket->readBlock(ptr, available);
  //kdDebug() << "Size of received data: " << size << endl;

  // Check if OK
  if(size > 0) {
    m_received += size;
    //kdDebug() << "New buffersize: " << m_received << endl;

    // Process all received packets
    while(m_received >= minsize) {
      packetsize = get_lcp3_pack_size(((t_lcp3_cmd*) m_inBuffer)->cmd);
      //kdDebug() << "Size of packet received: " << packetsize << endl;

      if(m_received >= packetsize) {
        // Process the received packet
        temp = (char*) malloc(packetsize);
        memcpy(temp, m_inBuffer, packetsize);
        // Remove processed size
        m_received -= packetsize;
        //kdDebug() << "Bytes left in buffer: " << m_received << endl;

        // Move remaining data to beginning of inBuffer
        if(m_received > 0) {
           ptr = (char*)m_inBuffer + packetsize;
           memmove(m_inBuffer, ptr, m_received);
        } else if(m_received < 0) {
           //kdDebug() << "Ooops! Memory management went bad! Received = " << m_received << endl;
           m_received = 0;
        }

        // Actually handle incomming
        processCmd(temp);
        free(temp);
        temp = 0;
      } else {
        // The whole packet hasn't been received yet!
        break;
      }
    }
  } else {
    //kdDebug() << "Error! Couldn't read from socket." << endl;
  }
}

void LcpIO::keepAlive() {
  SendQPost temp(CMD3_REFRESH);
  if(!temp.send(m_socket, m_selectedLine->lineId())) {
    emit cmdTimedOut();
    return;
  }
}

void LcpIO::processSendQ() {
  SendQPost *temp;

  // Get first item in list
  temp = m_sendQ->first();

  while(temp) { // Check all items in list
    if(temp->isTimedOut() && !temp->noTryLeft()) {
      if(!temp->send(m_socket, m_selectedLine->lineId())) {
        emit cmdTimedOut();
        return;
      }

      temp = m_sendQ->next(); // Next item in list
    } else if(temp->noTryLeft()) {
      m_sendQ->remove(); // Remove current item in list
      emit cmdTimedOut();
      return;
    } else {
      temp = m_sendQ->next(); // Next item in list
    }
  }

  // If the Q is empty we don't need to process it,
  // so we stop the resendTimer.
  if(m_sendQ->isEmpty()) {
    m_resendTimer->stop();
    m_keepAliveTimer->start(REFRESHINTERVAL);
  }
}

void LcpIO::connectionLost() {
  // We are disconnected
  cleanup();

  if((m_autoReconnectTries <= m_autoReconnectCount) && m_autoReconnectCount) {
    m_autoReconnectTimer->start(m_autoReconnectDelay * 1000, true);
  } else {
    emit connectionDown();
  }
}

void LcpIO::postConnectProcessing() {
  m_allLinesReceived = true;

  if(m_savedLineId > 0) {
    setSelectedLine(m_savedLineId);
  } else {
    m_selectedLine = m_lines->at(1);
    addToSendQ(CMD3_LINESEL);
  }

  addToSendQ(CMD3_GETALL); // Get status for all lines

  // Auto online?
  if(m_autoOnline) {
    this->addToSendQ(CMD3_ONLINE);
  }

  //kdDebug() << "Selected Line: " << m_selectedLine->name() << " [" << m_selectedLine->lineId() << "]" << endl;
  emit lineChanged(m_selectedLine->name());
}

void LcpIO::reconnect() {
  if(m_socket != 0) {
    delete m_socket;
  }

  m_autoReconnectTries++;

  m_socket = new KExtendedSocket(m_host, m_port);
  m_socket->enableRead(true);
  connect(m_socket, SIGNAL(readyRead()), this, SLOT(receiveData()));
  //kdDebug() << "Connecting to server." << endl;
  int conState = m_socket->connect();

  if(conState == 0) {
    addToSendQ(CMD3_PUBKEYREQ);
  } else {
    //kdDebug() << "Connecting to server failed with error: " << conState  << endl;
    cleanup();

    if((m_autoReconnectTries <= m_autoReconnectCount) && m_autoReconnectEnabled) {
      m_autoReconnectTimer->start(m_autoReconnectDelay * 1000, true);
    } else {
      emit connectionDown();
    }
  }
}

void LcpIO::redial() {
  m_autoRedialTries++;
  addToSendQ(CMD3_ONLINE);

  // Use two ISDN channels as default !!
  if(m_selectedLine->useTwoChanDefault()) {
    addToSendQ(CMD3_CHANADD);
  }
}

void LcpIO::connectToServer(const QString host, const uint port,
  const QString user, const QString passwd)
{
  if(m_clientStatus <= CLT_NOAUTH) {
    if(m_socket != 0) {
      delete m_socket;
    }

    m_user = user;
    m_passwd = passwd;
    m_host = host;
    m_port = port;
    m_socket = new KExtendedSocket(host, port);
    m_socket->enableRead(true);
    connect(m_socket, SIGNAL(readyRead()), this, SLOT(receiveData()));
    //kdDebug() << "Connecting to server." << endl;
    int conState = m_socket->connect();

    if(conState == 0) {
      m_autoReconnectTries = 0;
      addToSendQ(CMD3_PUBKEYREQ);
    } else {
      //kdDebug() << "Connecting to server failed with error: " << conState  << endl;
      cleanup();
      m_autoReconnectTimer->start(m_autoReconnectDelay * 1000, true);
    }
  }
}

void LcpIO::disconnectFromServer() {
  if(m_clientStatus == CLT_OFFLINE) {
    addToSendQ(CMD3_UNREG);
  }
}

void LcpIO::online() {
  // If we are offline then go online
  if(m_clientStatus == CLT_OFFLINE) {
    m_allowOffline = false;
    m_autoRedialTries = 0;
    addToSendQ(CMD3_ONLINE);

    // Use two ISDN channels as default !!
    if(m_selectedLine->useTwoChanDefault()) {
      addToSendQ(CMD3_CHANADD);
    }
  }
}

void LcpIO::offline() {
  // If we are online then go offline
  if(m_clientStatus == CLT_ONLINE || m_clientStatus == CLT_DIALING) {
    m_allowOffline = true;
    addToSendQ(CMD3_OFFLINE);

    // Are we aborting a online request...
    if(m_clientStatus == CLT_DIALING) {
      emit clientstatusChanged(CLT_ABORTING);
    }
  }
}

void LcpIO::setAutoOnline(bool ao) {
  m_autoOnline = ao;
}

void LcpIO::setSelectedLine(int lineId) {
  if(m_allLinesReceived) {
    //kdDebug() << "New selected line is: " << lineId << endl;
    m_selectedLine = findLine(lineId);
    addToSendQ(CMD3_LINESEL);
  } else {
    m_savedLineId = lineId;
  }
}

void LcpIO::getServerStatus() {
  addToSendQ(CMD3_STATREQ);
}

void LcpIO::toggleLineLock() {
  if(m_selectedLine->status() == CST_LOCKED_UP_CLT) {
    addToSendQ(CMD3_UNLOCK);
  } else if(m_selectedLine->status() == CST_UP_SERVER) {
    addToSendQ(CMD3_LOCK);
  }
}

void LcpIO::toggleISDNChan() {
  if(m_selectedLine->numChannels() == 1) {
    addToSendQ(CMD3_CHANADD);
  } else {
    addToSendQ(CMD3_CHANRM);
  }

  m_waitChanUpdate = true;
}

void LcpIO::shutdownServer() {
  addToSendQ(CMD3_SHUTDOWN);
}

#include "lcpio.moc"
